local Cube3 = _radiant.csg.Cube3
local Point3 = _radiant.csg.Point3
--local log = radiant.log.create_logger('meu_log')

local CustomHeightMapRenderer = class()

function CustomHeightMapRenderer:_add_foothills_to_region(region3, rect, height)
   local biome_name = stonehearth.world_generation:get_biome_alias()
   local colon_position = string.find (biome_name, ":", 1, true) or -1
   local mod_name_containing_the_biome = string.sub (biome_name, 1, colon_position-1)
   local fn = "_add_foothills_to_region_" .. mod_name_containing_the_biome
   if self[fn] ~= nil then
      --found a function for the biome being used, named:
      -- self:_add_foothills_to_region_<biome_name>(args,...)
      self[fn](self, region3, rect, height)
   else
      --there is no function for this specific biome, so call a copy of the original from stonehearth
      self:_add_foothills_to_region_original(region3, rect, height)
   end
end


function CustomHeightMapRenderer:_add_foothills_to_region_original(region3, rect, height)
   local terrain_info = self._biome:get_terrain_info()
   local foothills_step_size = terrain_info.foothills.step_size
   local plains_max_height = terrain_info.plains.height_max

   local has_grass = height % foothills_step_size == 0
   local soil_top = has_grass and height-1 or height

   self:_add_soil_strata_to_region(region3, Cube3(
         Point3(rect.min.x, 0,        rect.min.y),
         Point3(rect.max.x, soil_top, rect.max.y)
      ))

   if has_grass then
      local material = self._block_types.grass_hills
      if material == nil then
         material = self._block_types.grass
      end
      region3:add_unique_cube(Cube3(
            Point3(rect.min.x, soil_top, rect.min.y),
            Point3(rect.max.x, height,   rect.max.y),
            material
         ))
   end
end

function CustomHeightMapRenderer:_add_foothills_to_region_desert_transition_temperate(region3, rect, height)
   local terrain_info = self._biome:get_terrain_info()
   local foothills_step_size = terrain_info.foothills.step_size
   local plains_max_height = terrain_info.plains.height_max

   local has_grass = terrain_info.foothills.has_grass or height % foothills_step_size == 0
   local soil_top = has_grass and height-1 or height

   if terrain_info.foothills.has_desert and soil_top > terrain_info.foothills.has_desert.height-5 then
      region3:add_unique_cube(Cube3(
         Point3(rect.min.x, 0,        rect.min.y),
         Point3(rect.max.x, soil_top, rect.max.y),
         self._block_types.dirt_sand
      ))
   else
      self:_add_soil_strata_to_region(region3, Cube3(
         Point3(rect.min.x, 0,        rect.min.y),
         Point3(rect.max.x, soil_top, rect.max.y)
      ))
   end

   if has_grass then
      local material = self._block_types.grass_hills
      if material == nil then
         material = self._block_types.grass
      end
      if terrain_info.foothills.has_desert and height > terrain_info.foothills.has_desert.height-5 and height <= terrain_info.foothills.has_desert.height then
         material = self._block_types.sand
      end
      region3:add_unique_cube(Cube3(
            Point3(rect.min.x, soil_top, rect.min.y),
            Point3(rect.max.x, height,   rect.max.y),
            material
         ))
   end
end

return CustomHeightMapRenderer
